// C Source File
// Created 22/09/01; 10:39:30 a.m.

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization
#define SAVE_SCREEN           // Save/Restore LCD Contents
#include <tigcclib.h>         // Include All Header Files

short _ti89;                  // Produce .89Z File

#define LINK_PORT 		0x60000E
#define LINK_STATUS 	0x60000C
#define CLK_L_DAT_L 	0x0300
#define CLK_H_DAT_L 	0x0100
#define CLK_L_DAT_H 	0x0200
#define CLK_H_DAT_H 	0x0000
#define READ_DATA			0x0400
#define K_ESC					0x108

short _ti89;                  // Produce .89Z File
short _ti92plus;              // Produce .9XZ File

// <LinkPort>
volatile int *pLinkPort = (int*) LINK_PORT;
int *pLinkStatus = (int*) LINK_STATUS;
int iOldPortStatus = 0;
unsigned BitPeriod1 = 0;
unsigned BitPeriod1Tx = 0;
unsigned BitPeriod15 = 0;
unsigned char SerialDataRx = 0;
unsigned BaudRateBreakPoints[ 9 ] = { 110, 300, 1200, 2400, 4800, 
						  											9600, 19200, 38400, 57600 }; 
unsigned BaudRateIndex=5;

void OpenLinkPort ();
void CloseLinkPort();
void SetBaudRate();
void DoDelay( unsigned DelayTime );
void SendByte( unsigned char ByteToSend );
int GetByte(long unsigned TimeOutValue);


void SetBaudRate(){
	switch (BaudRateIndex){
		case 0: // 110
			BitPeriod1 = 11658;
			BitPeriod15 = 17475;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 1:	// 300
			BitPeriod1 = 4259;
			BitPeriod15 = 6397;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 2:	// 1200
			BitPeriod1 = 1048;
			BitPeriod15 = 1577;
			BitPeriod1Tx = BitPeriod1;
			break;
	
		case 3: // 2400
			BitPeriod1 = 515;
			BitPeriod15 = 778;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 4: // 4800
			BitPeriod1 = 247;
			BitPeriod15 = 362;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 5: // 9600
			BitPeriod1 = 113;
			BitPeriod15 = 162;
			BitPeriod1Tx = BitPeriod1;
			break;
			
		case 6: // 19200
			BitPeriod1 = 46;
			BitPeriod15 = 76;
			BitPeriod1Tx = 44;
			break;
			
		case 7: // 38400
			BitPeriod1 = 13;
			BitPeriod15 = 27;
			BitPeriod1Tx = 11;
			break;
			
		case 8: // 57600
			BitPeriod1 = 2;
			BitPeriod15 = 10;
			BitPeriod1Tx = 0;
			break;

	}
//	return;
}


int GetByte(long unsigned TimeOutValue){
	unsigned BitCycle;
	long unsigned IdleCounter;
	unsigned char ByteIn = 0;
	IdleCounter = TimeOutValue;
	OSSetSR (0x0700);						// Disable all interrupts, so our critical 
															// timing is pure and not affected
	*pLinkPort = CLK_L_DAT_H;
	while (1){
		if ( TimeOutValue ){
			if ( --IdleCounter == 0 ){
				break;
			}
		}
		if ((*pLinkPort & READ_DATA) == 0)		// Serial port going high (+V) (Start Bit)
		{
			DoDelay( BitPeriod15 ) ;						
			for (BitCycle = 0; BitCycle < 8; BitCycle++){
				if ( *pLinkPort & READ_DATA ){
					ByteIn |= (1 << BitCycle);
				}
				DoDelay( BitPeriod1 );
			}
			SerialDataRx = ByteIn;
			OSSetSR (0x0000);
			return 1;
		}	// End if ( Serial Active )
	} // End of while (1)
	OSSetSR (0x0000);
	return 0;
}	// End of GetByte()


void SendByte(unsigned char ByteToSend ){
	unsigned BitCycle;

	OSSetSR (0x0700);								// Disable all interrupts, so our critical 
	  															// timing is pure and not affected
	*pLinkPort = CLK_L_DAT_H;

	*pLinkPort = CLK_H_DAT_H;									// Send Start Bit			
	DoDelay( BitPeriod1Tx );
	
	for (BitCycle = 0; BitCycle < 8; BitCycle++){
		if ( ByteToSend & ( 1 << BitCycle ) ){
			*pLinkPort = CLK_L_DAT_H;
		}
		else{
			*pLinkPort = CLK_H_DAT_H;
		}
		DoDelay( BitPeriod1Tx );
	}
	*pLinkPort = CLK_L_DAT_H;
	DoDelay( BitPeriod1Tx );									// Send Stop Bit
	OSSetSR (0x0000);
	return;
}


void DoDelay( unsigned DelayTime ){
	int Temp;
	while ( DelayTime-- ){
		Temp = *pLinkStatus;
	}
	return;	
}

void OpenLinkPort (){
	iOldPortStatus = *pLinkStatus;
	*pLinkStatus = 0xFF00;
	return;
}

void CloseLinkPort(){
	*pLinkPort = 0x0000;
	*pLinkStatus = iOldPortStatus;
	return;
}
// </LinkPort>  Thank you, Josh Christensen !!!

#define origenX 18  // X grid origin 16
#define origenY 18  //Y grid origin 
#define scrollB 153 // Left Vertical Scroll Line 
#define scrollE scrollB+6	//Right Vertical Scroll Line 
#define delta 12    // Grid unit size in pixels
#define rows 6      // Number of total grid rows
#define columns 11  // Number of total grid columns
#define comlen 20   // Maximum length for comments
#define adlen 4			// Maximum length for addresses

#define numTools 18 // Number of total tools
#define noTool 20   // Non-tool symbols offset

#define watchX 62   // Watches X origin
#define watchY 35   // Watches Y origin

enum symbol{NOTHING=-1,ARROW,NEWFILE,OPEN,SAVE,RUNG,NEWBRA,DEL,XIC,XIO,OTE,
					OTL,OTU,OSR,COUNTER,TIMER,RESET,JMP,LABEL,
					WIRE=noTool,RUNGBEG,RUNGEND,BRABEG,BRAEND,BENDBEG,BENDEND,
					VERTBEG,VERTEND};
char *symbolName[11]={"XIC","XIO","OTE","OTL","OTU","OSR",
												"CNT","TMR","RES","JMP","LBL"};

// data structure for logic buffer
struct Cell{
	enum symbol typ;
	char *addr;
	//unsigned char value;  // value for timers or counters
	char *comment;//[comlen]; // "Here's a [20] string"
	enum Bool down;
	enum Bool osr;
};

struct row{
	int rung;
	struct Cell cell[columns];
	struct row *next;
};

struct row *ladder; 			// logic buffer
int entries=0;    			  // number of rows in ladder
int x=0,y=0;							// cursor coordinate
int winOffset=0;					// window offset
int watch[4]={0,0,0,0};		// watched value
unsigned char watchIndex[4]={0,1,0,1};

// I couldnt get these out of a global scope 
// without leaving my code FUBAR :{

int toolOffset=0;		// tool offset... en la mira para tronarmelo
enum Bool refreshMenu=FALSE;

char *tools[numTools]={ $(arrow),$(newfile),$(open),$(save),
												$(rung),$(newbra),$(del),$(xic),$(xio),
												$(ote),$(otl),$(otu),$(osr),$(counter),
												$(timer),$(reset),$(jmp),$(label) };

char *noTools[9]={	$(wire),$(rungbeg),$(rungend),$(brabeg),$(braend),
										$(bendbeg),$(bendend),$(vertbeg),$(vertend) };



//preferences
short showGrid=FALSE;
short showRungs=TRUE;


// juicy stuff :}

// this function was in the help file... thanx, Zeljko Juric !!!
short show_picvar (char *SymName, short i, short j, short Attr){
  SYM_ENTRY *sym_entry = SymFindPtr (SymName, 0);
  if (!sym_entry) return FALSE;
  if (peek (HToESI (sym_entry->handle)) != PIC_TAG) return FALSE;
  BitmapPut (i, j, HeapDeref (sym_entry->handle) + 2, ScrRect, Attr);
  return TRUE;
}

struct row *newRung(int rung){
	int k=0;
	char num[adlen],com[comlen];
	struct row *temp;
	temp=malloc(sizeof(struct row));
	temp->rung=rung;
	temp->next=NULL;
	temp->cell[0].typ=RUNGBEG;
	temp->cell[0].addr=malloc(sizeof(char[adlen]));
	strcpy(temp->cell[0].addr,"N-A");
	strcpy(com,"Rung #");
	sprintf(num,"%d",rung);
	strcat(com,num);
	temp->cell[0].comment=malloc(sizeof(char[comlen]));;
	strcpy(temp->cell[0].comment,com);
	
  for (k=1;k<columns;k++){
		temp->cell[k].typ=WIRE;		
		temp->cell[k].down=FALSE;
		temp->cell[k].osr=FALSE;
  }
	return(temp);
}

struct row *copyRow(int level){
	struct row *search=ladder;
	int k=0;
	if(search!=NULL){
		while((search->next!=NULL)&&(k<level)){
			k++;
			search=search->next;
		}
	}
	return(search);
}

void insertRung(struct row *search){
	char num[adlen],com[comlen];
	struct row *temp;
	if(search==NULL){
		ladder=newRung(0);
	}
	else{
		if(search==ladder){
			temp=newRung(0);
			temp->next=ladder;
			ladder=temp;
			temp=temp->next;
		}
		else{
			if(y+winOffset>=entries){
				while(search->next!=NULL){
					search=search->next;
				}
				temp=newRung(search->rung);
				search->next=temp;
			}
			else{
				temp=ladder;
				while((temp->next!=NULL)&&(temp->next->rung<search->rung)){
					temp=temp->next;
				}
				temp->next=newRung(search->rung);
				temp->next->next=search;
				temp=temp->next->next;
			}
		}
		while(temp!=NULL){
			if (temp->cell[0].typ==RUNGBEG){
				strcpy(com,"Rung #");
				sprintf(num,"%d",temp->rung);
				strcat(com,num);
				if(!strcmp(com,temp->cell[0].comment)){					
					strcpy(com,"Rung #");
					sprintf(num,"%d",temp->rung+1);
					strcat(com,num);
					temp->cell[0].comment=malloc(sizeof(char[comlen]));
					strcpy(temp->cell[0].comment,com);
				}
			}
			temp->rung++;
			temp=temp->next;
		}
	}
	entries++;
}

enum Bool isFree(struct row *test, short branch){
	short k;
	enum Bool output=TRUE;
	for(k=branch;k<=x;k++){
		if(test->cell[k].typ!=NOTHING)
			output=FALSE;
	}
	return(output);
}

struct row *newBranch(int rung){
	struct row *temp;
	int k=0;
	
	temp=malloc(sizeof(struct row));
	temp->rung=rung;
	temp->next=NULL;
	
  for (k=0;k<columns;k++){
		temp->cell[k].typ=NOTHING;		
  }
  return(temp);
}

void insertBranch(struct row *search, short branch){
	struct row *temp=NULL;
	int k=0;
  
	if(search!=NULL){
		switch(search->cell[branch].typ){
			case BENDBEG:	search->cell[branch].typ=RUNGBEG; break;
			case WIRE:	search->cell[branch].typ=BRABEG; break;
			default:break;
		}
		
		if(branch<columns-2){
			switch(search->cell[x].typ){
				case BENDEND:	search->cell[x].typ=RUNGEND; break;
				case WIRE:	 search->cell[x].typ=BRAEND; break;
				default:break;
			}
		}

		while((search->next!=NULL)&&(search->next->rung==search->rung)&& !isFree(search->next,branch)){
			search=search->next;
			switch(search->cell[branch].typ){
				case BENDBEG:	search->cell[branch].typ=RUNGBEG; break;
				case NOTHING:	search->cell[branch].typ=VERTBEG; break;
				default:break;
			}
			
			if(branch<columns-2){
				switch(search->cell[x].typ){
					case BENDEND:	search->cell[x].typ=RUNGEND; break;
					case NOTHING:	search->cell[x].typ=VERTEND; break;
					default:break;
				}
			}
		}
		if((search->next!=NULL)&&(search->next->rung==search->rung)){
			temp=search->next;
		}
		else{
			temp=search->next;
			search->next=newBranch(search->rung);
			search->next->next=temp;
			temp=search->next;
			entries++;
		}
		
		temp->cell[branch].typ=BENDBEG;
		
		if(branch<columns-2){
			for(k=branch+1;k<x;k++){
				temp->cell[k].typ=WIRE;
			}
			temp->cell[x].typ=BENDEND;
		}
		else{
			temp->cell[columns-1].typ=WIRE;
		}		
	}
}


struct Cell copyCell(){
	struct row *search=copyRow(y+winOffset);
	struct Cell notFound;
	notFound.typ = NOTHING;
	if(search==NULL){return(notFound);}
	return(search->cell[x]);
}

void insertCell(int tool){
	struct row *search=copyRow(y+winOffset);
	
	if(search!=NULL){
		char addStr[adlen], comStr[comlen];

		search->cell[x].typ=tool;
		
		search->cell[x].addr=malloc(sizeof(char[adlen]));
		search->cell[x].comment=malloc(sizeof(char[comlen]));
		switch(tool){
			case XIC: 
			case XIO: strcpy(addStr,"I:0"); 
								strcpy(comStr,"Input"); 
								break;
			case OTE: 
			case OTL: 
			case OTU: strcpy(addStr,"O:0");  
								strcpy(comStr,"Output"); 
								break;
			case OSR: strcpy(addStr,"B:0");  
								strcpy(comStr,"Flag"); 
								break;
			case COUNTER:
								strcpy(addStr,"C:0");    
								if(x==columns-1){
									strcpy(comStr,"Counter"); 
								}
								else{
									strcpy(comStr,"10"); 
								}
								break;
			case TIMER: 
								strcpy(addStr,"T:0");    
								if(x==columns-1){
									strcpy(comStr,"Timer"); 
								}
								else{
									strcpy(comStr,"100"); 
								}
								break;
			case RESET: strcpy(addStr,"C:0");   
								strcpy(comStr,"Reset"); 
								break;
			case JMP: 
			case LABEL: strcpy(addStr,"N-A");   
								strcpy(comStr,"Label 1"); 
								break;
		}
		strcpy(search->cell[x].addr,addStr);
		strcpy(search->cell[x].comment,comStr);
	}
}

void cursor(){
	if(x==-1){
		show_picvar ($(full), 0, origenY+y*delta, A_XOR);
		}else{
		show_picvar ($(full), origenX+x*delta, origenY+y*delta, A_XOR);
	}
}

void grid(){ // toggles grid
	int i,j;
	short attr=A_REVERSE;
	cursor();
	if(showGrid){attr=A_NORMAL;}
	for(i=origenX;i<160;i+=delta)
		for(j=origenY;j<100;j+=delta)
			DrawPix(i,j,attr);
	cursor();
}


void toolRefresh(){
	short k;
	for(k=0;k<rows;k++){
		show_picvar ($(blank), 0, origenY+k*delta, A_REPLACE);
	}	
	if(toolOffset==0){
		show_picvar (tools[0], 0, origenY, A_XOR);
	}
	else{
		show_picvar ($(up), 0, origenY, A_XOR);
	}
	for(k=1;k<rows-1;k++){
		show_picvar (tools[k+toolOffset], 0, origenY+k*delta, A_XOR);
	}
	if(toolOffset==(numTools-rows)){
		show_picvar (tools[numTools-1], 0, origenY+k*delta, A_XOR);
	}
	else{
		show_picvar ($(down), 0, origenY+k*delta, A_XOR);
	}
}

void refresh(){
	int i,j=0,type;
	struct row *temp=ladder;
	char num[adlen];
	clrscr();
	toolRefresh();
	//if (temp = NULL){return;}
  
  for(i=0;i<winOffset;i++){
	  temp=temp->next;
  }
  
  while((temp!=NULL)&((j-winOffset)<rows)){  
	  for (i=0;i<columns;i++){
	    type=temp->cell[i].typ;
	    if(type!=NOTHING){
		    if(type<noTool){
		  		show_picvar (tools[type], origenX+i*delta, origenY+j*delta, A_REPLACE);
		  	}
		  	else{
		  		show_picvar (noTools[type-noTool], origenX+i*delta, origenY+j*delta, A_REPLACE);
		  	}
		  }
	  }
		if ((temp->cell[0].typ==RUNGBEG)&(j<rows)&(showRungs)){
 			sprintf (num, "%d",temp->rung);
			DrawStr(origenX-1, 6+origenY+j*delta,num,A_NORMAL);
		}
	  if(temp->cell[0].typ==NOTHING){
	  	show_picvar ($(vertbeg), origenX, origenY+j*delta, A_REPLACE);
	  }
	  if(temp->cell[columns-1].typ==NOTHING)
	  	show_picvar ($(vertend), origenX+columns*delta, origenY+j*delta, A_REPLACE);
	  else
	  	show_picvar ($(rungend), origenX+columns*delta, origenY+j*delta, A_REPLACE);
	  	
	  temp=temp->next;
	  j++;
  }
  cursor();
  grid();
  refreshMenu=TRUE;
  
  DrawLine (14,16,14,93, A_THICK1); 
  DrawLine (scrollE,16,scrollE,93, A_NORMAL); 
  DrawLine (0,93,160,93, A_NORMAL);

}

void help(){
	int ye=9, dy=6;
	HANDLE dialog;
	dialog = DialogNewSimple (140, 90);
	
	DialogAddTitle (dialog, "Ti-PLC \"Help\" (1/3)", BT_OK, BT_NONE);
	
	FontSetSys(F_4x6);
	DialogAddText (dialog, 3, ye+=dy, "You should read the enclosed");
	DialogAddText (dialog, 3, ye+=dy, "manual, but here are the basics:");
	ye+=dy;
	DialogAddText (dialog, 3, ye+=dy, "1) Valid addresses:");
	ye+=dy;
	DialogAddText (dialog, 3, ye+=dy, "    Inputs:         I:0 through I:7");
	DialogAddText (dialog, 3, ye+=dy, "    Outputs:      O:0 through O:7");
	DialogAddText (dialog, 3, ye+=dy, "    Flags:          B:0 through B:7");
	DialogAddText (dialog, 3, ye+=dy, "    Counters:   C:0 through C:7");
	DialogAddText (dialog, 3, ye+=dy, "    Timers:       T:0 through T:7");
		
	DialogDo (dialog, CENTER, CENTER, NULL, NULL);
	HeapFree(dialog);
	FontSetSys(F_4x6);
	
	dialog = DialogNewSimple (140, 90);
	DialogAddTitle (dialog, "Ti-PLC \"Help\" (2/3)", BT_OK, BT_NONE);
	ye=9;
	FontSetSys(F_4x6);
	DialogAddText (dialog, 3, ye+=dy, "2) ALL actions go at the last");
	DialogAddText (dialog, 3, ye+=dy, "    (rightmost) cell!");
	ye+=dy;
	DialogAddText (dialog, 3, ye+=dy, "3) Everything not located at the");
	DialogAddText (dialog, 3, ye+=dy, "    last cell is considered as");
	DialogAddText (dialog, 3, ye+=dy, "    part of a condition.");
	ye+=dy;
	DialogAddText (dialog, 3, ye+=dy, "4) Labels go at the first");
	DialogAddText (dialog, 3, ye+=dy, "    cell (after rung numbers).");
	DialogDo (dialog, CENTER, CENTER, NULL, NULL);
	
	HeapFree(dialog);
	FontSetSys(F_4x6);
	dialog = DialogNewSimple (140, 90);
	DialogAddTitle (dialog, "Ti-PLC \"Help\" (3/3)", BT_OK, BT_NONE);
	ye=9;
	FontSetSys(F_4x6);
	
	DialogAddText (dialog, 3, ye+=dy, "5) To increment (or decrement) a ");
	DialogAddText (dialog, 3, ye+=dy, "    timer or counter, place");
	DialogAddText (dialog, 3, ye+=dy, "    its symbol as an action.");
	ye+=dy;
	DialogAddText (dialog, 3, ye+=dy, "6) To check for a certain value");
	DialogAddText (dialog, 3, ye+=dy, "    of a timer or counter, place");
	DialogAddText (dialog, 3, ye+=dy, "    its symbol within a condition.");
	ye+=dy;
	DialogAddText (dialog, 3, ye+=dy, "7) Mail me if you want to see more!");
	DialogDo (dialog, CENTER, CENTER, NULL, NULL);
	
	HeapFree(dialog);
	FontSetSys(F_4x6);
}

void about(){
	int ye=9, dy=6;
	HANDLE dialog;
	dialog = DialogNewSimple (140, 85);
	
	DialogAddTitle (dialog, "About Ti-PLC", BT_OK, BT_NONE);
	
	FontSetSys(F_4x6);
	DialogAddText (dialog, 3, ye+=dy, "           Ti-PLC, V1.05 ");
	DialogAddText (dialog, 3, ye+=dy, "            by Kuashio");
	ye+=dy;
	DialogAddText (dialog, 3, ye+=dy, "Comments and bug report go to");
	DialogAddText (dialog, 3, ye+=dy, "        kuashio@hotmail.com");
	ye+=dy;
	DialogAddText (dialog, 3, ye+=dy, "This is the last you'll ever see of");
	DialogAddText (dialog, 3, ye+=dy, "this program unless I get requests to");
	DialogAddText (dialog, 3, ye+=dy, "finish it. Read the enclosed readme.");
		
	DialogDo (dialog, CENTER, CENTER, NULL, NULL);
	
	HeapFree(dialog);
	FontSetSys(F_4x6);
}

void properties(){
	struct row *thisRow=copyRow(y+winOffset);
	struct Cell *thisCell=&(thisRow->cell[x]);
	HANDLE dialog, popUpUD, popUpRet;
	char reqStr[comlen+adlen+2], rungStr[4], 
				addStr[adlen], comStr[comlen];
	int ye=15, dY=10;
	int *index=malloc(sizeof(int[3]));
	
	
	index[0]=thisCell->down+1;
	
	popUpUD = PopupNew ("Up / Down", 80);
	PopupAddText (popUpUD,-1,"Up",1);
	PopupAddText (popUpUD,-1,"Down",2);
	
	popUpRet = PopupNew ("Timer Type", 80);
	PopupAddText (popUpRet,-1,"ON Timer",1);
	PopupAddText (popUpRet,-1,"Retentive",2);


	if(((thisCell->typ==COUNTER)||(thisCell->typ==TIMER))&&(x==columns-1)){
		dialog = DialogNewSimple (140, 60+dY);
	}
	else{
		dialog = DialogNewSimple (140, 60);
	}
	DialogAddTitle (dialog, "Cell Properties", BT_OK, BT_CANCEL);
	
	strcpy(reqStr,"Rung: ");
	sprintf (rungStr, "%d",thisRow->rung);
	strcat (reqStr,rungStr);
	DialogAddText (dialog, 3, ye, reqStr);
	
	if(thisCell->typ!=RUNGBEG){
		strcpy(reqStr,"Symbol: ");
		strcat (reqStr,symbolName[(thisCell->typ)-XIC]);
		DialogAddText (dialog, 50, ye, reqStr);
	}

	strcpy(reqStr,"Address: ");
	strcpy(addStr, thisCell->addr);
	if((thisCell->typ==RUNGBEG)&(x==columns-1)){
		strcat(reqStr,addStr);
		DialogAddText (dialog, 3, ye+=dY, reqStr);
	}
	else{
		DialogAddRequest (dialog, 3, ye+=dY, reqStr, 0, adlen-1, adlen);
	}

	if((thisCell->typ==COUNTER)&(x==columns-1)){
		DialogAddPulldown (dialog, 3, ye+=dY, "Count : ", popUpUD, 0);
	}
	
	if((thisCell->typ==TIMER)&(x==columns-1)){
		DialogAddPulldown (dialog, 3, ye+=dY, "Timer Type : ", popUpRet, 0);
	}
	
	if(((thisCell->typ==COUNTER)|(thisCell->typ==TIMER))&(x<columns-1)){
		strcpy(reqStr,"Preset Value: ");
		if(!strcmp(thisCell->comment,"Symbol")){
			strcpy(thisCell->comment,"10");
		}
	}
	else{
		if((thisCell->typ==JMP)|(thisCell->typ==LABEL)){
			strcpy(reqStr,"Label Name: ");
		}
		else{
			strcpy(reqStr,"Comments: ");
		}
	}
	strcpy(comStr,thisCell->comment);
	DialogAddRequest (dialog, 3, ye+=dY, reqStr, strlen(addStr)+1, comlen-1, 12);
	
	strcpy(reqStr,addStr);
	reqStr[strlen(addStr)]=' ';
	reqStr[strlen(addStr)+1]='\0';
	strcat(reqStr,comStr);
	reqStr[strlen(addStr)]='\0';
	
	if (DialogDo (dialog, CENTER, CENTER, reqStr, index) == KEY_ENTER){
		if(strlen(reqStr)==adlen-1){
			strncpy(thisCell->addr,reqStr,strlen(reqStr)+1);
			strcpy(thisCell->comment,strchr(reqStr,0)+1);
			thisCell->addr[0]=toupper(thisCell->addr[0]);
			if((thisCell->typ==COUNTER)||(thisCell->typ==TIMER)){
				if(x==columns-1){
					thisCell->down=index[0]-1;
				}
				else{
					if(thisCell->comment[0]==(char)0xAD)
						thisCell->comment[0]='-';
				}
			}
		}
		else{
			FontSetSys(F_4x6);
			if(x>0)
				DlgMessage ("Invalid Address", "Please enter a 3-character address", 
										BT_OK,BT_NONE);
		}
	}
	HeapFree(dialog);
	HeapFree(popUpRet);
	HeapFree(popUpUD);
	FontSetSys(F_4x6);
}

int stampLink(unsigned char byteToSend){
	int flag=0;
	SerialDataRx=5;
	OpenLinkPort();
	*pLinkPort = CLK_L_DAT_H;					// Setting Tx (white) low, inactive and
																		// setting Rx (red) line high so we can sense its state.

  SetBaudRate();
	
	SendByte(byteToSend);
	flag=GetByte(100000);
	CloseLinkPort();
	return(flag);
}

void drawRect(int a, int b, int c, int d){
  DrawLine(a,b,c,b,A_NORMAL);
  DrawLine(a,b,a,d,A_NORMAL);
  DrawLine(a,d,c,d,A_NORMAL);
  DrawLine(c,b,c,d,A_NORMAL);
}

void settings(){
	unsigned char k, orY=15, dY=9;
	HANDLE dialog, popUpC, popUpT, popUpBaud;
	char com[4], num[2];
	int *index=malloc(sizeof(int[5]));

	for(k=0;k<4;k++)
		index[k]=watchIndex[k]+1;
	index[4]=BaudRateIndex+1;
		
	popUpC = PopupNew ("Counter", 80);
	popUpT = PopupNew ("Timer", 80);
	for(k=0;k<8;k++){
		strcpy(com,"C:");
		sprintf(num,"%d",k);
		strcat(com,num);
		PopupAddText (popUpC,-1,com,k+1);
		
		strcpy(com,"T:");
		sprintf(num,"%d",k);
		strcat(com,num);
		PopupAddText (popUpT,-1,com,k+1);
	}
	
	popUpBaud = PopupNew ("Baud Rate", 90);
	for(k=0;k<=8;k++){
		sprintf(num,"%u",BaudRateBreakPoints[k]);
		PopupAddText (popUpBaud,-1,num,k+1);
	}
	
	dialog = DialogNewSimple (140, 90);
	DialogAddTitle (dialog, "Run Settings", BT_OK, BT_CANCEL);
	DialogAddPulldown (dialog, 3, orY+=dY, "1st Counter Watch : ", popUpC, 0);
	DialogAddPulldown (dialog, 3, orY+=dY, "2nd Counter Watch : ", popUpC, 1);
	DialogAddPulldown (dialog, 3, orY+=dY, "1st Timer Watch : ", popUpT, 2);
	DialogAddPulldown (dialog, 3, orY+=dY, "2nd Timer Watch : ", popUpT, 3);
	orY+=dY-2;
	DialogAddPulldown (dialog, 3, orY+=dY, "Emulator Baud Rate : ", popUpBaud, 4);

	if (DialogDo (dialog, CENTER, CENTER, NULL, index) == KEY_ENTER){
		for(k=0;k<4;k++)
			watchIndex[k]=index[k]-1;
		BaudRateIndex=index[4]-1;
		SetBaudRate();
	}
	free(index);
	HeapFree(popUpC);
	HeapFree(popUpT);
	HeapFree(popUpBaud);
	HeapFree(dialog);
	FontSetSys(F_4x6);
}

void drawPLC(){
	int k;
	char com[4], num[2];
	
	FontSetSys(F_4x6);

	drawRect(32, 6, 122, 87);
	
  DrawLine(20,9,32,9,A_NORMAL);
  DrawLine(20,84,32,84,A_NORMAL);
  DrawLine(20,84,20,9,A_NORMAL);
  
  DrawLine(122,9,134,9,A_NORMAL);
  DrawLine(122,84,134,84,A_NORMAL);
  DrawLine(134,84,134,9,A_NORMAL);
  
	for(k=0;k<=7;k++){
		strcpy(com,"I:");
		sprintf(num,"%d",k);
		strcat(com,num);
		DrawStr(34,13+9*k,com,A_NORMAL);
		
		strcpy(com,"O:");
		sprintf(num,"%d",k);
		strcat(com,num);
		DrawStr(110,13+9*k,com,A_NORMAL);
	}
	
	strcpy(com,"C:");
	DrawStr(watchX,watchY,com,A_NORMAL);
	DrawStr(watchX,watchY+9,com,A_NORMAL);
	com[0]='T';
	DrawStr(watchX,watchY+18,com,A_NORMAL);
	DrawStr(watchX,watchY+27,com,A_NORMAL);
	
	for(k=0;k<4;k++){
		sprintf(num,"%d",watchIndex[k]);
		DrawStr(watchX+6,watchY+9*k,num,A_NORMAL);
		DrawStr(watchX+11,watchY+9*k,"=",A_NORMAL);
	}
	for(k=0;k<8;k++){
		sprintf(num,"%d",k);
		DrawStr(46+k*8, 73, num, A_REPLACE);
	}	
	
	drawRect(watchX-19,6,watchX+47,watchY+52);//
	drawRect(watchX-19,70,watchX+47,30);//
	
	FontSetSys(F_8x10);
  DrawStr(51,10,"Ti-PLC",A_NORMAL);
	FontSetSys(F_4x6);
  DrawStr(58,22,"By Kuashio",A_NORMAL);
}


void flush(char ins, char outs, char flags){
	char watchStr[10];
	int k;
	
	for(k=0;k<4;k++){
		sprintf(watchStr,"%d",watch[k]);
	  DrawStr(watchX+16,watchY+k*9,"          ",A_REPLACE);
	  DrawStr(watchX+16,watchY+k*9,watchStr,A_NORMAL);
	}  
  
	FontSetSys(F_4x6);
	
	for(k=0;k<=7;k++){
		char dX=8;
		if((ins) & (0x01<<k))
			show_picvar ($(on), 23, 12+9*k, A_REPLACE);
		else
			show_picvar ($(off), 23, 12+9*k, A_REPLACE);
		
		if((outs) & (0x01<<k))
			show_picvar ($(on), 125, 12+9*k, A_REPLACE);
		else
			show_picvar ($(off), 125, 12+9*k, A_REPLACE);
			
		if((flags) & (0x01<<k))
			show_picvar ($(on), 45+k*dX, 79, A_REPLACE);
		else
			show_picvar ($(off), 45+k*dX, 79, A_REPLACE);
	}	
}


int keyAction(int tecla){ //gets an action from a key
  int output=0;
	if((tecla==337)|(tecla==344)|(tecla==340)|(tecla==338)){ //an arrow key
		cursor();
		switch (tecla){
			case 337: if(y==0){
									if((x==-1)&(toolOffset>0)){
										toolOffset--;
										toolRefresh();
									}
									if((x>-1)&(winOffset>0)){
										winOffset--;
										refresh();
										return(0);
									}
								}
								else{
									y--;
								} 
								break; //up
			case 344: x++; break; //rite
			case 340: if(y==rows-1){
									if((x==-1)&(toolOffset<numTools-rows)){
										toolOffset++;
										toolRefresh();
									}
									if((x>-1)&(winOffset<entries-1)){
										winOffset++;
										refresh();
										return(0);
									}
								}
								else{
									y++;
								} break; //down
			case 338: x--; break; //left
		}
		if (x<-1) x=-1;  // restrict cursor within window
		if (x>columns-1) x=columns-1;
		if (y<0) y=0;
		if (y>rows-1) y=rows-1;
		cursor();
	}
	switch (tecla){
		case 13://enter
			if(x==-1){
				if((y==0)&(toolOffset>0)){
					toolOffset--;
					toolRefresh();
				}
				else{
					if((y==rows-1)&(toolOffset<numTools-rows)){
						toolOffset++;
						toolRefresh();
					}
					else{
						output=-10;
					} // action on toolbar
				}
				show_picvar ($(full), 0, origenY+y*delta, A_XOR);
			}
			else{
				output=-11; // action on workspace
			}
		break;
		
		case 264://escape 
			output=116;
		break;
		case 4360:	// quit (?)
			output=16;
		break;
		
		case KEY_F7://F7
		case KEY_APPS://APPS
			output=265;
		break;
		case KEY_SWITCH:
		case KEY_MODE:
		case KEY_CLEAR:
			output=tecla;
		break;
	} 
	return (output);
}

void standby(){

	struct{
		unsigned char INS;
		unsigned char OUTS;
		unsigned char FLAGS;
		int C[8];
		int T[8];
		
	}myPLC;

	HANDLE exec_handle;
	HANDLE menu;
	HANDLE dialog=0;
	int tecla=0;
  int action=0;
  short tool=0;
  char branch=0;
  enum Bool emulate=FALSE;

	menu = MenuNew (2, 240, 18);
	MenuAddText (menu, 0, " File ", 1, 0);
//		MenuAddText (menu, 1, "New", 11, 0);
//		MenuAddText (menu, 1, "Open", 12, 0);
//		MenuAddText (menu, 1, "Save", 13, 0);
//		MenuAddText (menu, 1, "Save As", 14, 0);
//		MenuAddText (menu, 1, "Preferences", 15, 0);
		MenuAddText (menu, 1, "Quit", 16, 0);
			
	MenuAddText (menu, 0, " Edit ", 2, 0);
//		MenuAddText (menu, 2, "Cut", 21, 0);
//		MenuAddText (menu, 2, "Copy", 22, 0);
//		MenuAddText (menu, 2, "Paste", 23, 0);
//		MenuAddText (menu, 2, "Delete", 24, 0);
//		MenuAddText (menu, 2, "Properties", 25, 0);
		
	MenuAddText (menu, 0, " View ", 3, 0);
//		MenuAddText (menu, 3, "Go to Rung", 31, 0);
		MenuAddText (menu, 3, "Toggle Rung Numbers", 32, 0);
//		MenuAddText (menu, 3, "Toggle Addresses", 33, 0);
//		MenuAddText (menu, 3, "Toggle Names", 34, 0);
		MenuAddText (menu, 3, "Toggle Grid", 35, 0);
		
	MenuAddText (menu, 0, " Rung ", 4, 0);
		//MenuAddText (menu, 4, "Add Rung", 41, 0);
		//MenuAddText (menu, 4, "Delete Rung", 42, 0);
		//MenuAddText (menu, 4, "Add Branch", 43, 0);
		//MenuAddText (menu, 4, "Delete Branch", 44, 0);
		//MenuAddText (menu, 4, "Add Input", 45, 0);
		//MenuAddText (menu, 4, "Add Output", 46, 0);
		//MenuAddText (menu, 4, "Add Other", 47, 0);

	MenuAddText (menu, 0, " Run ", 5, 0);
		MenuAddText (menu, 5, "Simulate", 51, 0);
		MenuAddText (menu, 5, "Emulate", 52, 0);
		//MenuAddText (menu, 5, "Scan", 53, 0);
		MenuAddText (menu, 5, "Settings", 54, 0);
		
	MenuAddText (menu, 0, " Help ", 6, 0);
		MenuAddText (menu, 6, "Help", 61, 0);
		MenuAddText (menu, 6, "About", 62, 0);
	
	exec_handle = MenuBegin (HLock (menu), 0, 0, 0);

	while (action!=16){
		int clr;		
		struct row *disp;
		DrawStr(122,2," APPS",A_NORMAL);
		DrawStr(122,9," Tool:",A_NORMAL);
		show_picvar ($(blank), 145, 2, A_REPLACE);		
		show_picvar (tools[tool], 145, 2, A_REPLACE);
		tecla =ngetchx ();
		
	  	if ((tecla >= 268)&(tecla<=273)){
		  	action = MenuKey (exec_handle, tecla);  
			} 
	   	else{	   	
				action = keyAction(tecla);
			}
	   	if (action){ // if there's something to do...
	   		switch(action){
		   		struct Cell *search;
		   		case KEY_MODE: tool=numTools-1;
		   		case KEY_APPS:
		   							tool++;
		   							if (tool==numTools)
											tool=0;
										show_picvar ($(blank), 145, 2, A_REPLACE);		
										show_picvar (tools[tool], 145, 2, A_REPLACE);
		   							break;
		   		case KEY_CLEAR: tool=DEL+1;
		   		case KEY_SWITCH: 
		   							tool--;
		   							if (tool==-1)
											tool=numTools-1;
										show_picvar ($(blank), 145, 2, A_REPLACE);		
										show_picvar (tools[tool], 145, 2, A_REPLACE);
		   							break;					
		  		case 32: 	// Toggle Rung Numbers
		  							showRungs=!showRungs;
		  							refresh();
		  							break;		  		
		  							
		  		case 35: 	// Toggle grid
		  							showGrid=!showGrid;
		  							grid();
		  							break;
		  							
		  		case 54: 	// Settings...
		  							settings();
		  							break;		  							
		  		
		  		case 61: // Help
		  							help();
		  							break;				
		  							
		  		case 62: // About
		  							about();
		  							break;					
		  				  									  		
					case -10:{// action on toolbar (find out what to do)
							   		if((y==0) & (toolOffset==0)){tool=0;}
										if((y==rows-1) & (toolOffset==(numTools-rows))){tool=numTools-1;}
							   		if((y>0)&(y<rows-1)){tool=toolOffset+y;}
							   		
							   		if((tool==NEWFILE) |(tool==OPEN) |(tool==SAVE)){action=-11;}
										toolRefresh();
										show_picvar ($(full), 0, origenY+y*delta, A_XOR);
					}if(action==-10){break;}
					case -11:{// action on workspace							   		
										switch(tool){
											case ARROW:{// Properties
												search=&(copyRow(y+winOffset)->cell[x]);
											  if(((search->typ<WIRE)|(search->typ==RUNGBEG))&
											  		(y+winOffset<entries)&(y>=0))
												properties();
											}break;
											case NEWFILE:{  //new
												dialog = DialogNewSimple (140, 55);
												DialogAddTitle (dialog, "New Ladder", BT_YES, BT_NO);
												DialogAddText (dialog, 3, 20, "This will clear the workspace.");
												DialogAddText (dialog, 3, 27, "          Are you sure?");
												if (DialogDo (dialog, CENTER, CENTER, NULL, NULL) == KEY_ENTER){
													struct row *temp;	
													FontSetSys(F_4x6);
													tool=ARROW;
													toolOffset=0;
													if((entries!=0) || (ladder!=NULL)){
														temp=ladder;
														while(entries>1){
															temp=ladder;
															while (temp->next->next!=NULL){
																temp=temp->next;
															}
															free(temp->next);
															temp->next=NULL;
															entries--;
														}
														free(ladder);
													}
													ladder=NULL;
													entries=0;
													y=-1;
													x=-2;
													refresh();	
												}
												FontSetSys(F_4x6);
												HeapFree (dialog);
											}break;
											
											case OPEN:
											case SAVE:
												DlgMessage ("w00t!", 
       "Not yet implemented!\n\nSend me an email asking me to implement this feature.\n\nMy email is kuashio@hotmail.com\n\n
                                         The Author", 
       									BT_OK,BT_NONE);
												FontSetSys(F_4x6);
											break;
											case RUNG:{ // insert a warm new Rung
												if((copyRow(y+winOffset)->cell[0].typ==RUNGBEG) ||
													(y+winOffset>=entries)){
													insertRung(copyRow(y+winOffset));
													refresh();
												}
											}break;
											case NEWBRA:{ // insert a warm new Branch
												if((copyRow(y+winOffset)->cell[x].typ==WIRE) ||
													 (x<=columns-2)){
													if ((branch) && (x>branch+1)){
															insertBranch(copyRow(y+winOffset),branch);
															branch=0;
													}
													else{
														if(branch==columns-2){
															insertBranch(copyRow(y+winOffset),branch);
															branch=0;
														}
														else{
															branch=x;
														}
													}					
													refresh();
												}
											
											}break;
											case DEL:   //delete
												if((x==0)&(y+winOffset<entries)&
													(copyRow(y+winOffset)->cell[0].typ==RUNGBEG)){
													dialog = DialogNewSimple (140, 55);
													DialogAddTitle (dialog, "Delete Rung?", BT_YES, BT_NO);
													DialogAddText (dialog, 3, 20, "Are you sure you want to");
													DialogAddText (dialog, 3, 27, "delete this entire Rung?");
													if (DialogDo (dialog, CENTER, CENTER, NULL, NULL) == KEY_ENTER){
														int j;
														struct row *temp=ladder, *temp2=NULL;
														FontSetSys(F_4x6);
														if(temp==NULL){break;}
														if(entries==1){
															free(temp); 
															ladder=NULL;
															entries--;
														}
														else{
															for(j=0;j<y-1+winOffset;j++){
																temp=temp->next;
															}
															if((y+winOffset==0)&(entries>=1)){
																while((temp->next!=NULL)&&(temp->next->rung==temp->rung)){
																	ladder=ladder->next;
																	free(temp);
																	temp=ladder;
																	entries--;
																}
																ladder=ladder->next;
																free(temp);
																entries--;
															}												
															else{
																while((temp->next!=NULL)&&(temp->next->rung==temp->rung+1)){
																	temp2=temp->next;
																	temp->next=temp2->next;
																	free(temp2);
																	entries--;
																}
															}
															temp=temp->next;
														}
														while(temp!=NULL){
														char num[adlen],com[comlen];
															if (temp->cell[0].typ==RUNGBEG){
																strcpy(com,"Rung #");
																sprintf(num,"%d",temp->rung);
																strcat(com,num);
																if(!strcmp(com,temp->cell[0].comment)){					
																	strcpy(com,"Rung #");
																	sprintf(num,"%d",temp->rung+1);
																	strcat(com,num);
																	temp->cell[0].comment=malloc(sizeof(char[comlen]));
																	strcpy(temp->cell[0].comment,com);
																}
															}
															temp->rung--;
															temp=temp->next;
														}
														FontSetSys(F_4x6);
														HeapFree (dialog);
														refresh();
													}
													FontSetSys(F_4x6);
													break;													
												}
												else{ 
														tool=WIRE;
												}
											default: {
												search=&(copyRow(y+winOffset)->cell[x]);
												if((((signed)(search->typ))!=NOTHING
												&& (search->typ<RUNGBEG)
												&& (y+winOffset<entries)) 
												&& !((tool==LABEL) & (x!=1))
												&& !((tool==OTE) & (x!=columns-1))
												&& !((tool==OTL) & (x!=columns-1))
												&& !((tool==OTU) & (x!=columns-1))
												&& !((tool==RESET) & (x!=columns-1))
												&& !((tool==JMP) & (x!=columns-1))
												&& !((tool==XIC) & (x==columns-1))
												&& !((tool==XIO) & (x==columns-1))
												&& !((tool==OSR) & (x==columns-1))
												){
												
													dialog = DialogNewSimple (140, 55);
													DialogAddTitle (dialog, "This will delete the current symbol", BT_YES, BT_NO);
													DialogAddText (dialog, 3, 20, "Are you sure you want to");
													DialogAddText (dialog, 3, 27, "delete this symbol?");
													if (((search->typ) == WIRE) || (DialogDo (dialog, CENTER, CENTER, NULL, NULL) == KEY_ENTER)){
														show_picvar ($(full), origenX+x*delta, origenY+y*delta, A_XOR);
														show_picvar (tools[tool], origenX+x*delta, origenY+y*delta, A_REPLACE);
														if(showGrid){DrawPix(origenX+x*delta, origenY+y*delta,A_NORMAL);}
														insertCell(tool);
														show_picvar ($(full), origenX+x*delta, origenY+y*delta, A_XOR);
														FontSetSys(F_4x6);
														if(tool!=WIRE){properties();}
														refresh();
													}
													HeapFree (dialog);
													FontSetSys(F_4x6);
												}
												if(tool==WIRE){tool=DEL;}
											}
										}
					}break;
					case 52:emulate=TRUE;
					case 51:{ // RUN!!													
						struct row *temp;
						char outBuff=0, inBuff=0, inold=0, outold=0;
						short inkey=0;
						enum Bool value=TRUE, first;
						int k, j, thisRung;
						
						enum Bool exec(struct row *temp, int i, int j, char value, char ins){
							enum Bool output=FALSE;
							char val=value, num[5];
							if (value){
								if((i==columns-1)|((i==columns-2)&(temp->cell[i].typ==BRABEG))){
									output=TRUE;
								}
								else{
									switch (temp->cell[i].typ){
										case RUNGEND:
										case BENDEND: output|=exec(copyRow(j-1),i,j-1,value,ins); break;
										
										case XIC:
											switch((temp->cell[i]).addr[0]){
												case 'I':
													val=ins & (0x01<<(((temp->cell[i]).addr[2])-48));
												break;
												case 'O':
													val=myPLC.OUTS & (0x01<<(((temp->cell[i]).addr[2])-48));
												break;
												case 'B':
													val=myPLC.FLAGS & (0x01<<(((temp->cell[i]).addr[2])-48));
												break;
											}
											output|=exec(temp,i+1,j,val,ins);
											break;
										case XIO:
											switch((temp->cell[i]).addr[0]){
												case 'I':
													val=!(ins & (0x01<<(((temp->cell[i]).addr[2])-48)));
												break;
												case 'O':
													val=!(myPLC.OUTS & (0x01<<(((temp->cell[i]).addr[2])-48)));
												break;
												case 'B':
													val=!(myPLC.FLAGS & (0x01<<(((temp->cell[i]).addr[2])-48)));
												break;
											}
											output|=exec(temp,i+1,j,val,ins);
											break;
											
										case OSR:
													if(!temp->cell[i].osr){
														val=TRUE;
														myPLC.FLAGS |= (0x01<<(temp->cell[i].addr[2]-48));
													}							
													else{
														val=FALSE;
														myPLC.FLAGS &= ~(0x01<<(temp->cell[i].addr[2]-48));
													}													 
											temp->cell[i].osr=TRUE;
											output|=exec(temp,i+1,j,val,ins);
											break;
										case COUNTER:
											sprintf(num,"%d",myPLC.C[(int)temp->cell[i].addr[2]-48]);
											if(strcmp(num,temp->cell[i].comment))
												val=FALSE;
											else
												val=TRUE;
											output|=exec(temp,i+1,j,val,ins);
											break;
											
										case TIMER:
											sprintf(num,"%d",myPLC.T[(int)temp->cell[i].addr[2]-48]);
											if(strcmp(num,temp->cell[i].comment))
												val=FALSE;
											else
												val=TRUE;
											output|=exec(temp,i+1,j,val,ins);
											break;
											
										case BRABEG:
											output|=exec(temp,i+1,j,value,ins);
											output|=exec(temp->next,i,j+1,value,ins);
											break;
										
										default: 
											output|=exec(temp,i+1,j,value,ins);
											break;
									}
								}
							}
							else{
								if((i==columns-1)|((i==columns-2)&(temp->cell[i].typ==BRABEG))){
									output=FALSE;
								}
								else{
									output&=exec(temp,i+1,j,value,ins);
								}
								if (temp->cell[i].typ == OSR){				 
									temp->cell[i].osr=FALSE;
								}
							}
							return(output);
						}	
						// END OF exec()
							
						clrscr();						
						first=TRUE;
						myPLC.INS=0x00;
						myPLC.OUTS=0x00;
						myPLC.FLAGS=0x00;
						for(k=0;k<=7;k++){
							myPLC.C[k]=0;
							myPLC.T[k]=0;
						}
						watch[0]=myPLC.C[watchIndex[0]];
						watch[1]=myPLC.C[watchIndex[1]];
						watch[2]=myPLC.T[watchIndex[2]];
						watch[3]=myPLC.T[watchIndex[3]];
						drawPLC();
						flush(myPLC.INS,myPLC.OUTS,myPLC.FLAGS);
						
						while(inkey!=KEY_ESC){
							void *kbq = kbd_queue ();
							if(emulate){
								inkey=1;
								if(stampLink(myPLC.OUTS)){
									myPLC.INS=SerialDataRx;
								}
								else{
									DlgMessage ("LinkPort Malfunction", "Check your Link connection.", BT_OK,BT_NONE);
									FontSetSys(F_4x6);
									break;
								}
								OSdequeue (&inkey, kbq);
							}
							else{
								if (!OSdequeue (&inkey, kbq)){							
									inBuff=inkey-48;
									
									inold=myPLC.INS;
									outold=myPLC.OUTS;
									
									if((inBuff>=0) & (inBuff<=7))
										myPLC.INS^=(0x01<<inBuff);
								}
							}
									
							if(/*(inkey!=0)&*/(inkey!=KEY_ESC)){
								temp=ladder;
								j=0;
						/*
DrawStr(20,20,"ya!!!",A_NORMAL);
ngetchx();	*/

								while(temp!=NULL){
									enum Bool sepuede;
									sepuede=TRUE;
									thisRung=temp->rung;
									value=TRUE;
									value=exec(temp,1,j,value,myPLC.INS);
									
									do{	
										switch(temp->cell[columns-1].typ){
											case OTE:
												switch((temp->cell[columns-1]).addr[0]){
													case 'O':								
														if(value)
															outBuff|=(0x01<<(temp->cell[columns-1].addr[2]-48));									
														else
															outBuff&= ~(0x01<<(temp->cell[columns-1].addr[2]-48));
													break;
													case 'B':							
														if(value)
															myPLC.FLAGS|=(0x01<<(temp->cell[columns-1].addr[2]-48));								
														else
															myPLC.FLAGS&= ~(0x01<<(temp->cell[columns-1].addr[2]-48));
													break;
												}
											break;
											case OTL:
												switch((temp->cell[columns-1]).addr[0]){
													case 'O':								
														if(value)outBuff|=(0x01<<(temp->cell[columns-1].addr[2]-48));
													break;
													case 'B':
														if(value)myPLC.FLAGS|=(0x01<<(temp->cell[columns-1].addr[2]-48));
													break;
												}
											break;
											case OTU:
												switch((temp->cell[columns-1]).addr[0]){
													case 'O':
														if(value)outBuff&= ~(0x01<<(temp->cell[columns-1].addr[2]-48));
													break;
													case 'B':
														if(value)myPLC.FLAGS&= ~(0x01<<(temp->cell[columns-1].addr[2]-48));
													break;
												}
											break; 
											case COUNTER:
													if((value)& !(temp->cell[columns-1].osr)){
														if(temp->cell[columns-1].down)
															(myPLC.C[temp->cell[columns-1].addr[2]-48])--;
														else
															(myPLC.C[temp->cell[columns-1].addr[2]-48])++;
														temp->cell[columns-1].osr=TRUE;
													}
													if(!value){
														temp->cell[columns-1].osr=FALSE;
													}
											break;
											case TIMER:
													if(value){
														(myPLC.T[temp->cell[columns-1].addr[2]-48])++;
													}
													else{
														if(!temp->cell[columns-1].down){
															(myPLC.T[temp->cell[columns-1].addr[2]-48])=0;
														}
													}
											break;
											case RESET:
													if(value){
														if(temp->cell[columns-1].addr[0]=='C')
															(myPLC.C[temp->cell[columns-1].addr[2]-48])=0;
														if(temp->cell[columns-1].addr[0]=='T')
															(myPLC.T[temp->cell[columns-1].addr[2]-48])=0;
													}
											break;
											case JMP:
													if(value){
														char com[comlen];
														strcpy(com,temp->cell[columns-1].comment);
														temp=ladder;
														while(((temp!=NULL)&&(temp->cell[1].typ!=LABEL))|| 
																	((temp!=NULL) &&(strcmp(com,temp->cell[1].comment)))){
															temp=temp->next;
															j++;
														}
														sepuede=FALSE;
													}
											break;
											default: break;
										}										
											temp=temp->next;
											j++;
									}
									while((temp->cell[columns-2].typ == RUNGBEG ||
												 temp->cell[columns-2].typ == BENDBEG) && (sepuede));
									
									while((temp!=NULL)&&(temp->rung==thisRung)&&sepuede){
										temp=temp->next;
										j++;
									}
								}
											
								myPLC.OUTS=outBuff;
								
								watch[0]=myPLC.C[watchIndex[0]];
								watch[1]=myPLC.C[watchIndex[1]];
								watch[2]=myPLC.T[watchIndex[2]];
								watch[3]=myPLC.T[watchIndex[3]];
																
								flush(myPLC.INS,myPLC.OUTS,myPLC.FLAGS);
							}
						}
						refresh();
					} emulate=FALSE;
					break;
					case 116:{
									if((x>-5)&(y>-1)){
										x=-5;
										y=-1;
										refresh();
										break;
									} 
									action=16;
					}
					case 16:{ // quit
									dialog = DialogNewSimple (140, 55);
									DialogAddTitle (dialog, "Exit Ti-PLC", BT_YES, BT_NO);
									DialogAddText (dialog, 3, 20, "Are you sure you want to quit?");
									if (DialogDo (dialog, CENTER, CENTER, NULL, NULL) == KEY_ESC){
						   			action=0;
										FontSetSys(F_4x6);
						   		}
									HeapFree (dialog);
					}break;
	   		}
	   	}		
			for (clr=94;clr<100;clr++){
				DrawLine(0,clr,160,clr,A_REVERSE);
			}
	   	//  <Balloon Help>
			if(entries){				
				disp=copyRow(y+winOffset);
				if((x>0)&&(disp->cell[x].typ<WIRE)&(disp->cell[x].typ!=NOTHING)&(y+winOffset<entries)){
					int orX=1;					
					
					DrawStr(orX,95,"Symbol: ",A_REPLACE);
					DrawStr(orX+=28,95,symbolName[disp->cell[x].typ-XIC],A_REPLACE);
					orX+=17;
					DrawLine(orX,94,orX,100,A_REPLACE);	
					
					DrawStr(orX+=2,95,"Address: ",A_REPLACE);
					DrawStr(orX+=29,95,disp->cell[x].addr,A_REPLACE);
					orX+=14;
					DrawLine(orX,94,orX,100,A_REPLACE);	
					
					if(((disp->cell[x].typ==COUNTER)|(disp->cell[x].typ==TIMER))&(x<columns-1)){					
						DrawStr(orX+=2,95,"Preset: ",A_REPLACE);
						DrawStr(orX+=25,95,disp->cell[x].comment,A_REPLACE);
					}
					else{
						if((disp->cell[x].typ==JMP)|(disp->cell[x].typ==LABEL)){
							DrawStr(orX+=2,95,"Label: ",A_REPLACE);
							DrawStr(orX+=22,95,disp->cell[x].comment,A_REPLACE);
						}
						else{
							DrawStr(orX+=2,95,"Comment: ",A_REPLACE);
							DrawStr(orX+=34,95,disp->cell[x].comment,A_REPLACE);
						}
					}
				}
			}
			if(x==-1){
				char buffer[50];
				
				if(y!=5){
					switch(y+toolOffset){
						case ARROW:	strcpy(buffer,"Arrow Tool.  Shortcut Key: MODE");	break;
						case NEWFILE:	strcpy(buffer,"New Ladder");	break;
						case OPEN:
						case SAVE:	strcpy(buffer,"Not Yet Implemented"); break;
						case RUNG:	strcpy(buffer,"Add Rung");	break;
						case NEWBRA:	strcpy(buffer,"Add Branch");	break;
						case DEL:	strcpy(buffer,"Delete Tool.  Shortcut Key: CLEAR"); break;
						
						/*case XIC:	
						case XIO:	
						case OTE:	
						case OTL:	
						case OTU:	
						case OSR:	
						case COUNTER:	
						case TIMER:
						case RESET:	
						case JMP: 
							strcpy(buffer,"Symbol: ");	
							strcat(buffer,symbolName[y+toolOffset-XIC]);	
						break;*/
					}
				}
				if(((y!=5)&&(y+toolOffset<LABEL)&&(y+toolOffset>=XIC))||
					 ((y+toolOffset==LABEL)&&(y==5))){
					strcpy(buffer,"Symbol: ");	
					strcat(buffer,symbolName[y+toolOffset-XIC]);
				}
				
				if((toolOffset<numTools-6)&&(y==5)){	
					strcpy(buffer,"To switch to the next tool, press APPS");
				}
				if((toolOffset)&&(y==0)){					
					strcpy(buffer,"To switch to the previous tool, press 2nd+APPS");
				}
				DrawStr(1,95,buffer,A_REPLACE);
			}
	   	//  </Balloon Help>
	   	
			if(refreshMenu){
				MenuEnd (exec_handle);
				HeapUnlock (menu);
				exec_handle = MenuBegin (HLock (menu), 0, 0, 0);
				refreshMenu=FALSE;
				FontSetSys(F_4x6);
			}
	}
	MenuEnd (exec_handle);
	HeapUnlock (menu);
	HeapFree (menu);
	MenuUpdate ();
}

// Main Function
void _main(void){
	clrscr();
	drawPLC();
	
	watch[0]='K';
	watch[1]='u';
	watch[2]='a';
	watch[3]='s';	
	flush   ('h','i','o');
  ngetchx();
  clrscr();
	toolOffset=0;
	winOffset=0;
  y=-1;
  x=-5;  
  refresh();
  standby(); 
}
